package cz.cvut.ok.kapo.paa.batoh.app;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import cz.cvut.ok.kapo.paa.batoh.Solution;
import cz.cvut.ok.kapo.paa.batoh.engines.bf.ConfigurationsCruncher;
import cz.cvut.ok.kapo.paa.batoh.utils.ConfigurationException;
import cz.cvut.ok.kapo.paa.batoh.utils.ConfigurationReader;
import cz.cvut.ok.kapo.paa.batoh.utils.SolutionException;
import cz.cvut.ok.kapo.paa.batoh.utils.SolutionsWriter;

/**
 * Projede zadany adresar a pro vsechny obsazene soubory
 * spusti zadany engine.
 * @author karel
 */
public class DirectoryCruncher {
	public final static String LOGGER_NAME = "DIRECTORY_CRUNCHER_LOGGER";
	protected String dirIn, dirOut;
	protected Writer writer;
	public Logger log;
	
	/**
	 * 
	 * @param dirIn	vstupni adresar.
	 * @param dirOut adresar s vystupnimi soubory.
	 * @param writer writer, do ktereho se budou zapisovat statistiky.
	 */
	public DirectoryCruncher(String dirIn, String dirOut, Writer statWriter) {
		super();
		this.dirIn=dirIn;
		this.dirOut=dirOut;
		this.writer=statWriter;
		log = Logger.getLogger(DirectoryCruncher.LOGGER_NAME);
	}
	
	/**
	 * Spusti vypocet pres vsechny soubory v zadanem adreari.
	 *
	 */
	public void execute(){
		File fdi, fdo;
		fdi = new File(dirIn);
		fdo = new File(dirOut);
		
		if (!fdi.isDirectory() | !fdo.isDirectory()){
			log.warning("Input or output directory is not a folder! exitting");
			return;
		}
		FileReader fr;
		Writer fw;
		ConfigurationReader cr;
		SolutionsWriter sw;
		Vector<Solution> bfSols, shSols;

		try{
			writer.write("pocet;cas[ms];chyba[prumerna];chyba[maximalni]\n\r");
		}
		catch(IOException ex){
			log.log(Level.SEVERE, "Chyba pri zapisu do statistik!", ex);
		}		
		
		for (File fi: fdi.listFiles()){
			try {
				System.out.println("Soubor "+fi.getName());
				//inicializace
				fr = new FileReader(fi);
				cr = new ConfigurationReader(fr);
				cr.readConfigurations();
				
				//brutal force:
				fw = new FileWriter(fdo.getAbsolutePath()+File.separator+"BF"+fi.getName());
				ConfigurationsCruncher ccbf = new ConfigurationsCruncher(cr.getConfigurations());
				ccbf.execute();
				sw = new SolutionsWriter(fw);
				sw.setSolutions(bfSols=ccbf.getSolutions());
				sw.writeSolutios();
				fw.close();
				
				//simple heuristic:
				fw = new FileWriter(fdo.getAbsolutePath()+File.separator+"SH"+fi.getName());
				cz.cvut.ok.kapo.paa.batoh.engines.sh.ConfigurationsCruncher ccsh = new cz.cvut.ok.kapo.paa.batoh.engines.sh.ConfigurationsCruncher(cr.getConfigurations());
				ccsh.execute();
				sw = new SolutionsWriter(fw);
				sw.setSolutions(shSols=ccsh.getSolutions());
				sw.writeSolutios();
				fw.close();

				//statistika heuristiky
				long time=0;
				double maxErr=0.0, avgErr=0.0, tmp;
				for (int i=0;i<bfSols.size(); i++){
					Solution sb = bfSols.get(i);
					Solution sh = shSols.get(i);
					//	vypocet prumerneho casu
					time+=sb.getSolutionTime();
					//	vypocet max. a prumerne odchylky
					tmp = (sb.getSolutionCost()-sh.getSolutionCost())*100.0/sb.getSolutionCost();
					avgErr+=tmp;
					if (maxErr<tmp)
						maxErr=tmp;
				}
				time = time / bfSols.size();
				avgErr = avgErr / bfSols.size();
				
				writer.write(
						bfSols.get(0).getConfiguration().getItemsCount()+";"
						+time+";"
						+avgErr+";"
						+maxErr+"\r\n");
				writer.flush();

			}
			catch(IOException ex){
				log.log(Level.SEVERE, "Chyba pri otevirani souboru: "+ fi.getName(), ex);
			}
			catch(ConfigurationException ex){
				log.log(Level.SEVERE, "Chyba pri nacitani konfigurace!", ex);
			}
			catch(SolutionException ex){
				log.log(Level.SEVERE, "Chyba pri ukladani vysledku!", ex);
			}
		}
	}
	

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		FileWriter sw=null;
		DirectoryCruncher dc=null;
		
		if (args.length<2){
			System.out.println("Chybi parametry: <adresar.vstupni> [adresar.vystupni] <stats.soubor>\n\r");
			return;
		}
		else if (args.length<3){
			try {
				sw = new FileWriter(args[1]);
			}
			catch(IOException ex){
				System.out.println("Chyba pri otevirani souboru se statistikami!");
			}
			dc=new DirectoryCruncher(args[0],args[0],sw);
			dc.execute();
		}
		else if (args.length<4){
			try {
				sw = new FileWriter(args[2]);
			}
			catch(IOException ex){
				System.out.println("Chyba pri otevirani souboru se statistikami!");
			}
			dc=new DirectoryCruncher(args[0],args[1],sw);
			dc.execute();
		}

		System.out.println("Hotovo");

	}

}
